import display
from vector import Vector
import collections
import bisect
from OpenGL.GL import *
from OpenGL.GLU import *
import nanothreads
import eventnet.driver
import time

import draw
import texture
import Image


class Graph(draw.Shape):
    """
    Graphs are collections of nodes which can be rendered. Graphs can contain other graphs as nodes.
    """
    def __init__(self, items=[]):
        draw.Shape.__init__(self)
        self.items = []
        self.position = (0.0,0.0)
        for item in items:
            self.items.append(item)
            
    def add(self, item, position=(0,0)):
        item.container = self
        self.items.append(item)
        
    def remove(self, item):
        self.items.remove(item)
        
    def render(self):
        for item in self.items:
            display._screen.blit(item, item.position)
    
    def collide(self, position, w=3, h=3):
        """
        Returns all objects in the group which cover the virtual coordinates
        specified in the position argument. Generally used for simple collision detection,
        as the game engine should use virtual coordinates, not screen coordinates.
        """
        position = position[0],-position[1]
        pos =  (Vector(position) / display._screen.ratio) + Vector((display._screen.width, display._screen.height))/2
        return self.pick(pos, w, h)
            
    def pick(self, position, w=3, h=3):
        """
        Returns all objects in the group which cover the screen coordinates
        specified in the position argument. Generally used for detecting which Nodes have
        been clicked with the mouse, as the mouse uses screen coordinates.
        """
        glInitNames()
        glSelectBuffer(256)
        glRenderMode(GL_SELECT)
        glMatrixMode(GL_PROJECTION)
        glPushMatrix()
        glLoadIdentity()
        viewport = glGetIntegerv(GL_VIEWPORT)
        n_w,n_h = display._screen.dimensions

        gluPickMatrix(position[0],viewport[3]-position[1],w,h,viewport)
        glOrtho(-n_w,n_w,-n_h,n_h,1,0)
        glMatrixMode(GL_MODELVIEW)
        for index, i in enumerate(self.items):
            glPushName(index)
            display._screen.blit(i, i.position)
            glPopName()
        glMatrixMode(GL_PROJECTION)
        glPopMatrix()
        glMatrixMode(GL_MODELVIEW)
        glFlush()
        
        items = []
        for hit in glRenderMode(GL_RENDER):
            items = items + [self.items[i] for i in hit[2]]
        items.reverse()
        return items

        
