from vector import Vector
from collections import deque
import pygame

class RingBuffer(deque):
    def __init__(self, size):
        deque.__init__(self)
        self.size = size
    
    def append(self, item):
        deque.append(self, item)
        if len(self) > self.size:
            self.popleft()


class Entity(object):
    def __init__(self):
        self.__position = Vector([0.0,0.0])
        self.positions = RingBuffer(10)
        self.force = Vector((0,0))
        self._t = pygame.time.get_ticks()
        self.decay = 0.1
    
    def stop(self):
        self.force = Vector((0,0))
        
    def get_position(self):
        t = pygame.time.get_ticks()
        time_increment = (t - self._t) / 1000.0
        self._t = t
        
        inc = self.force * time_increment
        self.positions.append(self.__position)
        self.__position += inc
        
        self.force *= (self.decay ** time_increment)
        
        return self.__position
    position = property(get_position)

    def apply_force(self, vector):
        self.force += vector
