from OpenGL.GL import *
from OpenGL.GLU import *

import display
from vector import Vector
import draw

import Image
import ImageOps
import JpegImagePlugin
import PngImagePlugin
import os
import glob
import math


MAX_TEXTURE_SIZE = 2048, 2048

powers_of_two = [int(math.pow(2,i)) for i in xrange(100)]
def match_closest_power_of_two(value):
    for i in powers_of_two:
        if i >= value:
            return i


class Texture(object):
    """
    A simple Texture. A texture must be used by a SiGL.draw.Image object in order to be blitted.
    """
    def __init__(self, surface):
        self.surface = surface
        self.size = Vector(surface.size) + (1,1)
        self.n_size = self.size * display._screen.ratio



class TexturePack(object):
    """
    A texture pack combines multiple images into one OpenGL texture.
    The texture pack dimensions must be n**2 for some integer n. eg (512,512), (512,1024), (1024,1024).
    This restriction is an OpenGL requirement.
    """
    def __init__(self,filter=False):
        self.surfaces = []
        self.filter = filter

    def add(self, surface, build=True):
        """
        Add a PIL image to the texture pack. Returns a SiGL sprite which can be blitted.
        """
        t = Texture(surface)
        self.surfaces.append(t)
        t.pack = self
        if build: self.build()
        return t

    def build(self):
        """
        Builds all loaded PIL images into OpenGL textures, ready for blitting. Raises ValueError if images cannot fit into texture pack.
        """
        x = 0
        y = 0
        my = 0
        width = height = 0
        offset = Vector((x,y))
        paste_ops = []

        for area, texture in reversed(sorted([(i.size.y,i) for i in self.surfaces])):
            block = Vector(texture.size)

            if block.y > my: my = block.y

            if (x + block.x) >= MAX_TEXTURE_SIZE[0]:

                x = 0
                y += my
                if (x + block.x) > MAX_TEXTURE_SIZE[0]: raise ValueError, "Cannot fit texture into pack, horizontal limit exceeded."
                if y + block.y > MAX_TEXTURE_SIZE[1]: raise ValueError, "Cannot fit texture into pack, vertical limit exceeded."

            if x+block.x >= width: width = x+block.x
            if y+block.y >= height: height = y+block.y

            paste_ops.append((texture, (x,y), block))
            x += block.x

        #print width,height
        self.size = match_closest_power_of_two(width),match_closest_power_of_two(height)
        self.surface = Image.new("RGBA", self.size)
        self.id = glGenTextures(1)
        #print "Created Texture:", self.id

        for texture,xy,block in paste_ops:
            block = block
            #print self.surface.size,texture,xy,block
            surface = ImageOps.flip(texture.surface)
            self.surface.paste(surface, xy)
            texture.offset = xy
            l,t = self.normal_point(xy)
            r,b = self.normal_point((block+xy))
            texture.texels = (l,b),(r,t)
            texture.build = self.build
            
        self.data = self.surface.tostring()
        self.load()

    def load(self):
        filter_type = GL_NEAREST
        if self.filter: filter_type = GL_LINEAR
        glDeleteTextures(self.id)
        self.id = glGenTextures(1)
        glBindTexture(GL_TEXTURE_2D, self.id)
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,filter_type)
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,filter_type)
        try:
            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, self.size[0], self.size[1], 0, GL_RGBA, GL_UNSIGNED_BYTE, self.data)
        except Exception, e:
            self.surface.save("bad_texture.png")
            raise Exception, e

    def normal_point(self, xy):
        if xy[0] == 0:
            x = 0.0
        else:
            x = 1.0/(float(self.size[0])/xy[0])
        if xy[1] == 0:
            y = 0.0
        else:
            y = 1.0/(float(self.size[1])/xy[1])
        return x,y

    def __del__(self):
        try:
            glDeleteTextures(self.id)
            #print "Cleaned up Texture:",self.id
        except AttributeError:
            pass
        except NamerError:
            pass

def build_pack(dir):
    pack = TexturePack()
    d = {}
    for filename in glob.glob(dir):
        print 'adding', filename
        key = os.path.split(filename)[-1]
        d[key] = pack.add(Image.open(filename))
    pack.build()
    return d
