from OpenGL.GL import *
import surface
import video
import math
import _psyco

@_psyco.proxy
def isometric_grid(width, height, size):
    h = math.tan((math.pi / 180) * 30) * (size/2)
    y = 0
    t = 0
    while y <= height:
        x = 0
        if t:
            x = (size/2)
        t = not(t)
        while x <= width:
            yield x,y
            x += size
        y += h


class DisplayList(object):
    """
    An OpenGL display list abstraction. If the list uses any surfaces, it is important
    to pass them in via the references keyword, so that they are garbage collected.
    """
    def __init__(self, references=None):
        self._references = references
        self._id = glGenLists(1)
        self.rotation = 0.0
        self.scale = 1.0
        self.alpha = 1.0
        self.centre_point = 0.0,0.0
        self.captured = False

    def centre(self):
        self.centre_point = (-self.size[0]/2,-self.size[1]/2)

    def capture(self):
        video._screen.bind_texture(0)
        glNewList(self._id, GL_COMPILE)

    def release(self):
        self.captured = True
        glEndList()

    def render(self):
        glCallList(self._id)

    def __del__(self):
        if glDeleteLists:
            glDeleteLists(self._id, 1)

class Tile(object):
    def __init__(self, surface, texels, size):
        self.surface = surface
        self.size = size
        self.texels = texels
        self.rotation = 0.0
        self.scale = 1.0
        self.alpha = 1.0
        self.centre_point = 0.0,0.0
        self.display_list = DisplayList()
        self.centre()

    def centre(self):
        self.centre_point = (-self.size[0]/2,-self.size[1]/2)

    @_psyco.proxy
    def render(self):
        #if self.display_list.captured:
        #    self.display_list.render()
        #    return
        #self.display_list.capture()
        w,h = self.size
        x,y = self.centre_point
        glPushMatrix()
        glTranslatef(x,y,0)
        video._screen.bind_texture(self.surface._id)
        glColor4f(1.0, 1.0, 1.0, self.alpha)
        glBegin(GL_QUADS)
        self.draw(w,h)
        glEnd()
        glPopMatrix()
        #self.display_list.release()

    @_psyco.proxy
    def draw(self, w, h):
        x,y,tw,th = self.texels
        glTexCoord2f(x,y);glVertex2f(0, 0)
        glTexCoord2f(x+tw,y);glVertex2f(w, 0)
        glTexCoord2f(x+tw,y+th);glVertex2f(w, h)
        glTexCoord2f(x,y+th);glVertex2f(0, h)


class Tiles(object):
    """
    Loads a pygame surface into a Tileset, accessible by index.
    """
    def __init__(self, surf, count, size):
        self.surface = surface.convert(surf)
        self.size = size
        self.count = count
        self.cache = {}

    @_psyco.proxy
    def __getitem__(self, i):
        if i in self.cache: return self.cache[i]
        y = i // self.count[0]
        x = (i - y * self.count[0]) - 1
        w,h = self.size[0], self.size[1]
        tw = 1.0/self.count[0]
        th = 1.0 /self.count[1]
        t = Tile(self.surface, (x*tw,y*th,tw,th), self.size)
        self.cache[i] = t
        return t

