import SiGL
import SiGL.gui
import SiGL.grid
from SiGL.base import Mix
from SiGL.vector import Vector
import eventnet.driver
import Image, ImageFont, ImageDraw

import scene_loader
import sys
import random
import nanothreads

import pygame.mixer

from lib import effectors
from lib import mobiles
import time

class Mask(object):
    def __init__(self, image):
        self.image = image
        
    def empty(self, position):
        position = position[0],-position[1]
        pos =  (Vector(position) / SiGL.display._screen.ratio) + Vector((self.image.size))/2
        try:
            r,g,b = self.image.getpixel(pos)
        except:
            return False
        if r < 10:
            return True
        return False
        
        
class Rabbit(SiGL.draw.FrameSet):
    def __init__(self, textures):
        self.textures = textures
        d = dict(tl=textures['tl.png'], tr=textures['tl.png'], br=textures['br.png'], bl=textures['bl.png'],rip=textures['rip.png'])
        SiGL.draw.FrameSet.__init__(self, d, position=(1,1))
        self.hp = 20.0
        
    def __repr__(self):
        return "<%s hp=%s>" % (self.__class__.__name__, self.hp)
        
class Strider(object):
    def __init__(self, textures,mask):
        self.mask = mask
       
        self.down = SiGL.draw.Animation(sorted([textures[i] for i in textures if "down" in i]), fps=10)
        self.down.index = int(random.uniform(0, len(self.down.textures)-1))
        
        
        self.delegate = self.down
        self.hp = 20
    
    def update(self):
        def change():
            while True:
                position = Vector([random.uniform(-0.5,0.5),random.uniform(-0.5,0.5)])
                
                
                if self.mask.empty(position+self.position):
                    self.rotation = position.angle()
                    position = position + self.position
                    th = SiGL.actions.Move(self, position=position, duration=random.uniform(1,10)).execute()
                    nanothreads.register_exit(th, change)
                    break
            
        change()
        
    def __getattr__(self, key):
        return getattr(self.delegate, key)
        
        

class Map(eventnet.driver.Handler):
    def __init__(self):
        eventnet.driver.Handler.__init__(self)
        self.capture()
        self.graph = SiGL.graph.Graph()
        self.init_map()
        self.centre = 0,0
        self.textures = SiGL.texture.build_pack("tiles/effects/*.png")
        self.sounds = SiGL.sound.build_pack("sounds/*.ogg")
        self.sounds.update(SiGL.sound.build_pack("sounds/*.wav"))
        self.mask = Mask(Image.open("images/mask.jpg"))
        self.Effector = effectors.LightningEffector
        self.effects = SiGL.graph.Graph()
        self.sprites = SiGL.graph.Graph()
        self.herd = SiGL.graph.Graph()
        self.graph.add(self.herd)
        self.graph.add(self.effects)
        self.init_sprites()
        self.init_herd()
        
    def EVT_PLAY_SOUND(self, event):
        self.sounds[event.name].play()
        
    def EVT_DAMAGE(self, event):
        hits = self.herd.collide(event.position, w=event.radius, h =event.radius)
        for hit in hits:
            if hit.hp >= 0:
                hit.hp -= event.damage
                if hit.hp < 0:
                    self.herd.remove(hit)
                    rip = SiGL.draw.Image(self.statics['rip.png'], position=hit.position,color=[1,1,1,0])
                    self.effects.add(rip)
                    nanothreads.defer_for(1, SiGL.actions.FadeIn(rip,duration=2.0).execute)
                    nanothreads.defer_for(0.1, self.effects.add, SiGL.draw.Animation(self.blood, position=hit.position, loop=False,fps=30))
                    self.sounds['splat.ogg'].play()
                    eventnet.driver.post('ADJUST_SCORE',amount=10)
                    
                
        
    def init_herd(self):
        rabbits = SiGL.texture.build_pack("images/sprites/strider/*.png")
        for i in xrange(100):
            rabbit = Strider(rabbits,self.mask)
            while True:
                position = random.uniform(-2,2),random.uniform(-2,2)
                if self.mask.empty(position): break
          
            rabbit.position = position
            rabbit.update()
            self.herd.add(rabbit)
            
        
    def init_sprites(self):
        statics = SiGL.texture.build_pack("images/sprites/static/*.png")
        hawk = mobiles.Hawk(statics, self.sounds)
        self.sprites.add(hawk.image)
        hawk.enable()
        self.statics = statics
        for i in xrange(20):
            cloud = mobiles.Cloud(statics, self.sounds)
            self.sprites.add(cloud.shadow)
            self.sprites.add(cloud.image)
            cloud.enable()
        
        self.graph.add(self.sprites)
        
        blood = SiGL.texture.build_pack("images/sprites/blood/*.png")
        self.blood = [blood[i] for i in sorted(blood.keys())]
    
    def init_map(self):
        tp = SiGL.texture.TexturePack()
        texmap = Image.open("images/landscape.png")
        texmap = tp.add(texmap)
        tp.build()
        
        #mtp = SiGL.texture.TexturePack()
        #mtex = mtp.add(Image.open("images/mask.jpg"))
        #mtp.build()
        
        img = SiGL.draw.Image(texmap)
        #mask = SiGL.draw.Image(mtex)
        #mask.position = 0,0
        img.position = 0,0
        self.graph.add(img)
        #self.graph.add(mask)
        
    def effect(self, position):
        effect = self.Effector(self.textures, self.sounds, position)
        eventnet.driver.post('ADJUST_SCORE', amount=-effect.penalty)
        effect.container = self
        self.effects.add(effect.graph)
        if effect is effectors.MeteorEffector:
            eventnet.driver.post('REMOVE_METEOR')
            
        
        
class Herd(object):
    def __init__(self):
        self.graph = SiGL.graph.Graph()
        


class GameFrame(SiGL.gui.ScrollFrame):
    def __init__(self):
        SiGL.gui.ScrollFrame.__init__(self,speed=1)
        
        self.engine = Map()
        self.interface = SiGL.graph.Graph()
        self.map = self.engine.graph
        self.interface_textures = SiGL.texture.build_pack("images/interface/*.png")
        self.rabbits = 100
        self.cursor = SiGL.draw.Image(self.interface_textures['finger_pointer.png'], position=(0,0))
        self.cursor.centre_point=(0,0)
        
        self.add(self.map)
        self.add(self.interface)
        self.add(self.cursor)
        self.end_scene = scene_loader.load("xml/congrats.xml")
        
        
        self.init_interface()
        pygame.mouse.set_visible(False)
        
    def EVT_KeyDown(self, event):
        if event.key == 27:
            raise SystemExit
        
    def register(self):
        song = pygame.mixer.music.load('music/ingame.ogg')
        pygame.mixer.music.set_endevent(2000)
        @eventnet.driver.handle('Unknown')
        def replay(event):
            pygame.mixer.music.play()
        pygame.mixer.music.play()


        
        
    def init_interface(self):        
        
        hud = SiGL.draw.Image(self.interface_textures['base_hud.png'], color=[1,1,1,1])
        hud.position = (0,-0.77)
        
        self.interface.add(hud)
        
        d = {
                'happy':self.interface_textures['professor_happy.png'],
                'worried':self.interface_textures['professor_worried.png'],
                'angry':self.interface_textures['professor_angry.png']
            }
        
        self.professor = SiGL.draw.FrameSet(d,position=(-0.8,-0.77),scale=1)
        self.interface.add(self.professor)
        
        self.interface.add(SiGL.draw.Image(self.interface_textures["sml_logo.png"], position=[-1.05,-0.75]))
        
        string_pack = SiGL.text.StringPack()
        self.numbers = [string_pack.add(i) for i in "0123456789-"]
            
        self.score = 0
        self.score_image = self.build_score(0)
        self.interface.add(self.score_image)
        self.init_buttons()
    
    def build_score(self, score_value):
        score = SiGL.graph.Graph()
        score.position= (-0.58,-0.805)
        x = 0
        for c in str(score_value):
            if c == "-": c = 10
            img = SiGL.draw.Image(self.numbers[int(c)].texture, position=(x,0))
            x += 0.049
            score.add(img)
        return score
        
    def init_buttons(self):
        def onoff(*textures):
            return dict(on=textures[0], off=textures[1])
        
        d = onoff(self.interface_textures['button_lightning_on.png'], self.interface_textures['button_lightning.png'])
        self.button_lightning = SiGL.draw.FrameSet(d, position=(0,-0.72),default="on")
        self.interface.add(self.button_lightning)
        d = onoff(self.interface_textures['button_tornado_on.png'], self.interface_textures['button_tornado.png'])
        self.button_tornado = SiGL.draw.FrameSet(d, position=(0.25,-0.72),default="off")
        self.interface.add(self.button_tornado)
        d = onoff(self.interface_textures['button_quake_on.png'], self.interface_textures['button_quake.png'])
        self.button_quake = SiGL.draw.FrameSet(d, position=(0.5,-0.72),default="off")
        self.interface.add(self.button_quake)
        d = onoff(self.interface_textures['button_meteor_on.png'], self.interface_textures['button_meteor.png'])
        self.button_meteor = SiGL.draw.FrameSet(d, position=(0.75,-0.72),default="off")
        self.interface.add(self.button_meteor)
        
    def EVT_ADJUST_SCORE(self, event):
        if event.amount == 10:
            self.rabbits -= 1
            if self.rabbits <= 0:
                print "No More Rabbits."
                eventnet.driver.post('GAMEOVER', score=self.score + 10)
                self.graph.add(self.end_scene.graph)
                self.end_scene.play()
                nanothreads.defer_for(10, eventnet.driver.post, 'show_menu')
                
                
        self.score += event.amount
        if self.score < 0: nanothreads.defer_for(11, eventnet.driver.post, 'PROF_ANGRY')
        
        self.interface.remove(self.score_image)
        self.score_image = self.build_score(self.score)
        self.interface.add(self.score_image)
        
        
    def EVT_PROF_HAPPY(self, event):
        self.professor.set('happy')

    def EVT_PROF_WORRIED(self, event):
        self.professor.set('worried')
        
    def EVT_PROF_ANGRY(self, event):
        self.professor.set('angry')
        
    def EVT_MouseMotion(self, event):
        SiGL.gui.ScrollFrame.EVT_MouseMotion(self, event)
        position = SiGL.gui.xy(event.pos)
        self.cursor.position = position
        
        
    def EVT_MouseButtonDown(self, event):
        if event.button == 1:
            hits = self.interface.pick(event.pos)
            if hits:
                hit = hits[0]
                buttons = self.button_lightning, self.button_tornado, self.button_quake, self.button_meteor
                def off():
                    eventnet.driver.post("PLAY_SOUND", name="select.ogg")
                    for b in buttons: b.set("off")
                        
                if hit == self.button_lightning:
                    off()
                    self.button_lightning.set("on")
                    self.engine.Effector = effectors.LightningEffector
                elif hit == self.button_tornado:
                    off()
                    self.button_tornado.set("on")
                    self.engine.Effector = effectors.TyphoonEffector
                elif hit == self.button_quake:
                    off()
                    self.button_quake.set("on")
                    self.engine.Effector = effectors.QuakeEffector
                elif hit == self.button_meteor:
                    off()
                    self.button_meteor.set("on")
                    self.engine.Effector = effectors.MeteorEffector
                    
            else:
                self.engine.effect(SiGL.gui.xy(event.pos)-self.map.position)
                if self.engine.Effector is effectors.MeteorEffector:
                    self.interface.remove(self.button_meteor)
                    self.button_lightning.set("on")
                    self.engine.Effector = effectors.LightningEffector
                    eventnet.driver.post('PROF_HAPPY')
                nanothreads.defer_for(10, eventnet.driver.post, 'PROF_WORRIED')
        elif event.button == 4:
            pass
        elif event.button == 5:
            pass
        
    def EVT_MouseButtonUp(self, event):
        pass

    def update(self, position):
        self.engine.position = position
        self.map.position = tuple(position)


class IntroFrame(SiGL.gui.Frame):
    def __init__(self):
        SiGL.gui.Frame.__init__(self)
        self.scene = scene_loader.load("xml/intro.xml")
        self.graph.add(self.scene.graph)
        
        self.stop_th = nanothreads.defer_for(89, self.EVT_KeyDown, None)
    
    def EVT_KeyDown(self, event):
        self.scene.stop()
        nanothreads.terminate(self.stop_th)
        eventnet.driver.post('show_menu')
        
        
    def register(self):
        self.scene.play()
        
        
class MenuFrame(SiGL.gui.Frame):
    def __init__(self):
        SiGL.gui.Frame.__init__(self)
        self.scene = scene_loader.load("xml/menu.xml")
        self.graph.add(self.scene.graph)
    
    def EVT_MouseButtonDown(self, event):
        hits = self.scene.graph.pick(event.pos)
        if hits:
            hit = hits[0]
            if self.scene.images["start"] is hit:
                SiGL.actions.FadeOut(self.scene.images["start"]).execute()
                SiGL.actions.FadeOut(self.scene.images["quit"]).execute()
                th = SiGL.actions.FadeIn(self.scene.images["pleasewait"]).execute()
                nanothreads.register_exit(th, eventnet.driver.post, 'start_game')
                
            if self.scene.images["quit"] is hit:
                raise SystemExit
    def register(self):
        pygame.mouse.set_visible(True)
        self.scene.play()


def main():
    app = SiGL.gui.Application((800,600),icon="quido.png",caption="Quido van Killnhurtz' Failed Experiment")

    frame = IntroFrame()
    app.activate_frame(frame)
    
    @eventnet.driver.handle('start_game')
    def start_game(event):
        
        if hasattr(app, 'gameframe'):
            frame = app.gameframe
        else:
            frame = GameFrame()
        app.active_frame.scene.stop()
        time.sleep(5)
        app.activate_frame(frame)
        app.gameframe = frame
        
    @eventnet.driver.handle('show_menu')
    def start_menu(event):
        if hasattr(app, 'menuframe'):
            frame = app.menuframe
        else:
            frame = MenuFrame()
        app.activate_frame(frame)
    
    
    
    app.loop()

if __name__ == "__main__": main()





